@@ -35,7 +35,7 @@
 #include "knowndrives.h"
 #include "config.h"
 
-const char *ataprint_c_cvsid="$Id: ataprint.c,v 1.129 2004/02/06 03:52:02 ballen4705 Exp $"
+const char *ataprint_c_cvsid="$Id: ataprint.c,v 1.135 2004/02/15 21:15:20 ballen4705 Exp $"
 ATACMDNAMES_H_CVSID ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
 
 // for passing global control variables
@@ -120,12 +120,15 @@ void printswap(char *output, char *in, unsigned int n){
  * to produce errors).  If many more are to be added then this function
  * should probably be redesigned.
  */
-char *construct_st_er_desc(unsigned char CR, unsigned char FR,
-                                 unsigned char ST, unsigned char ER)
-{
+char *construct_st_er_desc(struct ata_smart_errorlog_struct *data) {
+  unsigned char CR=data->commands[4].commandreg;
+  unsigned char FR=data->commands[4].featuresreg;
+  unsigned char ST=data->error_struct.status;
+  unsigned char ER=data->error_struct.error_register;
   char *s;
   char *error_flag[8];
-  int i;
+  int i, print_lba=0;
+
   /* If for any command the Device Fault flag of the status register is
    * not used then used_device_fault should be set to 0 (in the CR switch
    * below)
@@ -146,6 +149,7 @@ char *construct_st_er_desc(unsigned char CR, unsigned char FR,
     error_flag[2] = "ABRT";
     error_flag[1] = "NM";
     error_flag[0] = "obs";
+    print_lba=1;
     break;
   case 0x25:  /* READ DMA EXT */
   case 0xC8:  /* READ DMA */
@@ -157,6 +161,7 @@ char *construct_st_er_desc(unsigned char CR, unsigned char FR,
     error_flag[2] = "ABRT";
     error_flag[1] = "NM";
     error_flag[0] = "obs";
+    print_lba=1;
     break;
   case 0x30:  /* WRITE SECTOR(S) */
   case 0xC5:  /* WRITE MULTIPLE */
@@ -166,6 +171,7 @@ char *construct_st_er_desc(unsigned char CR, unsigned char FR,
     error_flag[3] = "MCR";
     error_flag[2] = "ABRT";
     error_flag[1] = "NM";
+    print_lba=1;
     break;
   case 0xA0:  /* PACKET */
     /* Bits 4-7 are all used for sense key (a 'command packet set specific error
@@ -228,13 +234,14 @@ char *construct_st_er_desc(unsigned char CR, unsigned char FR,
     error_flag[2] = "ABRT";
     error_flag[1] = "NM";
     error_flag[0] = "obs";
+    print_lba=1;
     break;
   default:
     return NULL;
   }
 
-  /* 100 bytes -- that'll be plenty (OK, this is lazy!) */
-  if (!(s = (char *)malloc(100)))
+  /* 256 bytes -- that'll be plenty (OK, this is lazy!) */
+  if (!(s = (char *)malloc(256)))
     return s;
 
   s[0] = '\0';
@@ -253,11 +260,34 @@ char *construct_st_er_desc(unsigned char CR, unsigned char FR,
     for (i = 7; i >= 0; i--)
       if ((ER & (1 << i)) && (error_flag[i])) {
         if (count++ > 0)
-          strcat(s, ", ");
+           strcat(s, ", ");
         strcat(s, error_flag[i]);
       }
   }
 
+  // If the error was a READ or WRITE error, print the Logical Block
+  // Address (LBA) at which the read or write failed.
+  if (print_lba) {
+    char tmp[128];
+    int lba;
+
+    // bits 24-27: bits 0-3 of DH
+    lba   = 0xf & data->error_struct.drive_head;
+    lba <<= 8;
+    // bits 16-23: CH
+    lba  |= data->error_struct.cylinder_high;
+    lba <<= 8;
+    // bits 8-15:  CL
+    lba  |= data->error_struct.cylinder_low;
+    lba <<= 8;
+    // bits 0-7:   SN
+    lba  |= data->error_struct.sector_number;
+
+    // print LBA, and append to print string
+    snprintf(tmp, 128, " at LBA = 0x%08x = %d", lba, lba);
+    strcat(s, tmp);
+  }
+
   return s;
 }
 
@@ -721,8 +751,8 @@ int ataPrintLogDirectory(struct ata_smart_log_directory *data){
 
 // returns number of errors
 int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
-  int i,j,k;
-  
+  int k;
+
   pout("SMART Error Log Version: %d\n", (int)data->revnumber);
   
   // if no errors logged, return
@@ -770,13 +800,15 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
     char *st_er_desc;
 
     // The error log data structure entries are a circular buffer
-    i=(data->error_log_pointer+k)%5;
-    
+    int j, i=(data->error_log_pointer+k)%5;
+    struct ata_smart_errorlog_struct *elog=data->errorlog_struct+i;
+    struct ata_smart_errorlog_error_struct *summary=&(elog->error_struct);
+
     // Spec says: unused error log structures shall be zero filled
-    if (nonempty((unsigned char*)&(data->errorlog_struct[i]),sizeof(data->errorlog_struct[i]))){
+    if (nonempty((unsigned char*)elog,sizeof(*elog))){
       // Table 57 of T13/1532D Volume 1 Revision 3
       char *msgstate;
-      int bits=data->errorlog_struct[i].error_struct.state & 0x0f;
+      int bits=summary->state & 0x0f;
       switch (bits){
       case 0x00: msgstate="in an unknown state";break;
       case 0x01: msgstate="sleeping"; break;
@@ -793,28 +825,23 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
       // See table 42 of ATA5 spec
       PRINT_ON(con);
       pout("Error %d occurred at disk power-on lifetime: %d hours\n",
-             (int)(data->ata_error_count+k-4), (int)data->errorlog_struct[i].error_struct.timestamp);
+             (int)(data->ata_error_count+k-4), (int)summary->timestamp);
       PRINT_OFF(con);
       pout("  When the command that caused the error occurred, the device was %s.\n\n",msgstate);
       pout("  After command completion occurred, registers were:\n"
            "  ER ST SC SN CL CH DH\n"
            "  -- -- -- -- -- -- --\n"
            "  %02x %02x %02x %02x %02x %02x %02x",
-           (int)data->errorlog_struct[i].error_struct.error_register,
-           (int)data->errorlog_struct[i].error_struct.status,
-           (int)data->errorlog_struct[i].error_struct.sector_count,
-           (int)data->errorlog_struct[i].error_struct.sector_number,
-           (int)data->errorlog_struct[i].error_struct.cylinder_low,
-           (int)data->errorlog_struct[i].error_struct.cylinder_high,
-           (int)data->errorlog_struct[i].error_struct.drive_head);
+           (int)summary->error_register,
+           (int)summary->status,
+           (int)summary->sector_count,
+           (int)summary->sector_number,
+           (int)summary->cylinder_low,
+           (int)summary->cylinder_high,
+           (int)summary->drive_head);
       // Add a description of the contents of the status and error registers
       // if possible
-      st_er_desc = construct_st_er_desc(
-        data->errorlog_struct[i].commands[4].commandreg,
-        data->errorlog_struct[i].commands[4].featuresreg,
-        data->errorlog_struct[i].error_struct.status,
-        data->errorlog_struct[i].error_struct.error_register
-      );
+      st_er_desc = construct_st_er_desc(elog);
       if (st_er_desc) {
         pout("  %s", st_er_desc);
         free(st_er_desc);
@@ -824,7 +851,7 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
            "  CR FR SC SN CL CH DH DC   Timestamp  Command/Feature_Name\n"
            "  -- -- -- -- -- -- -- --   ---------  --------------------\n");
       for ( j = 4; j >= 0; j--){
-        struct ata_smart_errorlog_command_struct *thiscommand=&(data->errorlog_struct[i].commands[j]);
+        struct ata_smart_errorlog_command_struct *thiscommand=elog->commands+j;
         
         // Spec says: unused data command structures shall be zero filled
         if (nonempty((unsigned char*)thiscommand,sizeof(*thiscommand)))
@@ -1352,7 +1379,7 @@ int ataPrintMain (int fd){
       failuretest(OPTIONAL_CMD, returnval|=FAILSMART);
     }
     if (ataReadErrorLog(fd, &smarterror)){
-      pout("Smartctl: SMART Errorlog Read Failed\n");
+      pout("Smartctl: SMART Error Log Read Failed\n");
       failuretest(OPTIONAL_CMD, returnval|=FAILSMART);
     }
     else {
