@@ -27,7 +27,7 @@
 #include "utility.h"
 #include "config.h"
 
-const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.85 2004/02/11 02:18:21 ballen4705 Exp $"
+const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.87 2004/02/16 18:56:59 pjwilliams Exp $"
                                 ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID UTILITY_H_CVSID;
 
 #define MODEL_STRING_LENGTH                         40
@@ -144,9 +144,9 @@ const drivesettings knowndrives[] = {
       "Please see http://www.geocities.com/dtla_update/index.html#rel",
     NULL, NULL, NULL
   },
-  { // IBM Deskstar 40GV & 75GXP series (TXAOA5AA firmware)
+  { // IBM Deskstar 40GV & 75GXP series (A5AA/A6AA firmware)
     "(IBM-)?DTLA-30[57]0[123467][05]$",
-    "^TXAOA5AA$",
+    "^T[WX][123468A]OA[56]AA$",
     NULL, NULL, NULL, NULL
   },
   { // IBM Deskstar 40GV & 75GXP series (all other firmware)
@@ -458,6 +458,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // TOS MK3019GAXB SUN30G
+    "^TOS MK3019GAXB SUN30G$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // Seagate Barracuda U Series 20410
     "^ST320410A$",
     ".*",
@@ -528,6 +533,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // Western Digital Caviar AC12500, AC24300, AC25100, AC36400, AC38400
+    "^WDC AC(125|243|251|364|384)00",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // Western Digital Caviar SE family
   /* Western Digital drives with this comment all appear to use Attribute 9 in
    * a  non-standard manner.  These entries may need to be updated when it
